@@ -26,6 +26,7 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
+import inspect
 import math
 import random
 import time
@@ -45,6 +46,10 @@ CACHE_PRUNE_TIME = 8000
 _colliders = []
 _collision_checkers = []
 
+# Set of objects that are active, to avoid looping through inactive
+# objects needlessly.
+_active_objects = set()
+
 
 class cache(object):
 
@@ -106,11 +111,6 @@ def _scale(surface, width, height):
 
 def _get_blend_flags(blend_mode):
     # Return the appropriate Pygame flags for the given blend mode.
-    if blend_mode is not None and blend_mode in (sge.BLEND_RGB_SCREEN,
-                                                 sge.BLEND_RGBA_SCREEN):
-        w = "Screen blend mode not supported. Normal blending used instead."
-        warnings.warn(w)
-
     pygame_flags = {
         sge.BLEND_RGBA_ADD: pygame.BLEND_RGBA_ADD,
         sge.BLEND_RGBA_SUBTRACT: pygame.BLEND_RGBA_SUB,
@@ -127,6 +127,30 @@ def _get_blend_flags(blend_mode):
     return pygame_flags
 
 
+def _screen_blend(dest, source, dest_x, dest_y, alpha=False):
+    dest.lock()
+    source.lock()
+    for y in six.moves.range(source.get_height()):
+        if 0 <= dest_y + y < dest.get_height():
+            for x in six.moves.range(source.get_width()):
+                if 0 <= dest_x + x < dest.get_width():
+                    dc = dest.get_at((dest_x + x, dest_y + y))
+                    sc = source.get_at((x, y))
+
+                    def blended_component(c1, c2):
+                        return int(255 - (((255 - c1) / 255) *
+                                          ((255 - c2) / 255) * 255))
+
+                    r = blended_component(dc.r, sc.r)
+                    g = blended_component(dc.g, sc.g)
+                    b = blended_component(dc.b, sc.b)
+                    a = blended_component(dc.a, sc.a) if alpha else dc.a
+
+                    dest.set_at((dest_x + x, dest_y + y), pygame.Color(r, g, b, a))
+    dest.unlock()
+    source.unlock()
+
+
 def _set_mode():
     # Set the mode of the screen based on self.width, self.height,
     # and self.fullscreen.
@@ -369,63 +393,6 @@ def _get_polygon_sprite(points, fill, outline, outline_thickness, anti_alias):
     return sprite
 
 
-def _get_rectangle_collision_areas(x, y, w, h):
-    # Get a list of collision areas a rect is in.
-    room = sge.game.current_room
-    area_size = room.rd["collision_area_size"]
-    room_areas = room.rd["collision_areas"]
-    areas_x_start = int(math.floor(x / area_size))
-    areas_x_num = int(math.ceil(w / area_size) + 1)
-    areas_y_start = int(math.floor(y / area_size))
-    areas_y_num = int(math.ceil(h / area_size) + 1)
-    areas = []
-
-    for i in six.moves.range(areas_x_start, areas_x_start + areas_x_num):
-        for j in six.moves.range(areas_y_start, areas_y_start + areas_y_num):
-            if (i >= 0 and j >= 0 and room_areas and i < len(room_areas) and
-                    j < len(room_areas[0])):
-                areas.append((i, j))
-            elif None not in areas:
-                areas.append(None)
-
-    return areas
-
-
-def _get_others(areas, other=None):
-    room = sge.game.current_room
-    others = []
-
-    for area in areas:
-        if area is not None:
-            i, j = area
-            room_area = room.rd["collision_areas"][i][j]
-        else:
-            room_area = room.rd["collision_area_void"]
-
-        for ref in room_area:
-            obj = ref()
-            if obj is not None:
-                if other is None:
-                    others.append(obj)
-                elif isinstance(other, sge.StellarClass):
-                    if obj is other:
-                        others.append(obj)
-                elif isinstance(other, (list, tuple)):
-                    if obj in other:
-                        others.append(obj)
-                elif other in sge.game.objects:
-                    if obj is sge.game.objects[other]:
-                        others.append(obj)
-                else:
-                    try:
-                        if isinstance(obj, other):
-                            others.append(obj)
-                    except TypeError:
-                        pass
-
-    return others
-
-
 def _get_hat(joystick, hat):
     # Return the position of a joystick HAT.
     if (joystick is not None and joystick < len(game_joysticks) and
@@ -496,6 +463,14 @@ def o_update(self, time_passed, delta_mult):
         self.image_index += int(self.rd["anim_count"] / self.rd["frame_time"])
         self.rd["anim_count"] %= abs(self.rd["frame_time"])
 
+        if self.sprite is not None:
+            while self.image_index >= self.sprite.frames:
+                self.image_index -= self.sprite.frames
+                self.event_animation_end()
+            while self.image_index < 0:
+                self.image_index += self.sprite.frames
+                self.event_animation_end()
+
     # Alarms
     activated_alarms = []
     for a in self.alarms:
@@ -511,6 +486,75 @@ def o_update(self, time_passed, delta_mult):
         self.event_update_position(delta_mult)
 
 
+def o_update_object_areas(self):
+    room = sge.game.current_room
+    if room is not None and self in room.objects:
+        x = self.bbox_left
+        y = self.bbox_top
+        w = self.bbox_width
+        h = self.bbox_height
+
+        if self.sprite is not None:
+            x = min(x, self.x - self.image_origin_x)
+            y = min(y, self.y - self.image_origin_y)
+            w = max(w, self.sprite.width)
+            h = max(h, self.sprite.height)
+
+        my_areas = r_get_rectangle_object_areas(room, x, y, w, h)
+    else:
+        my_areas = set()
+
+    for area in my_areas ^ self.rd["object_areas"]:
+        if area in my_areas:
+            if area is None:
+                area = (None, None)
+            i, j = area
+            if (i is not None and j is not None and
+                    i < len(room.object_areas) and
+                    j < len(room.object_areas[i])):
+                oa = room.object_areas[i][j].copy()
+                oa.add(self)
+                room.object_areas[i][j] = oa
+            else:
+                if i is not None or j is not None:
+                    e = "An object area existed in a {} object, but not in the room!".format(
+                        self.__class__.__name__)
+                    e += "\nAttempted area: ({}, {})".format(i, j)
+                    x = len(room.object_areas)
+                    y = len(room.object_areas[0]) if x else 0
+                    e += "\nAvailable areas: {}x{}".format(x, y)
+                    warnings.warn(e)
+
+                oa = room.object_area_void.copy()
+                oa.add(self)
+                room.object_area_void = oa
+        else:
+            if area is None:
+                area = (None, None)
+            i, j = area
+            if (i is not None and j is not None and
+                    i < len(room.object_areas) and
+                    j < len(room.object_areas[i])):
+                oa = room.object_areas[i][j].copy()
+                oa.discard(self)
+                room.object_areas[i][j] = oa
+            else:
+                if i is not None or j is not None:
+                    e = "An object area existed in a {} object, but not in the room!".format(
+                        self.__class__.__name__)
+                    e += "\nAttempted area: ({}, {})".format(i, j)
+                    x = len(room.object_areas)
+                    y = len(room.object_areas[0]) if x else 0
+                    e += "\nAvailable areas: {}x{}".format(x, y)
+                    warnings.warn(e)
+
+                oa = room.object_area_void.copy()
+                oa.discard(self)
+                room.object_area_void = oa
+
+    self.rd["object_areas"] = my_areas
+
+
 def o_update_collision_lists(self):
     global _colliders
     global _collision_checkers
@@ -525,15 +569,6 @@ def o_update_collision_lists(self):
             collision_checkers = _collision_checkers[:]
             collision_checkers.append(self)
             _collision_checkers = collision_checkers
-
-        if self.collision_precise:
-            my_areas = _get_rectangle_collision_areas(
-                self.mask_x, self.mask_y, len(self.mask),
-                len(self.mask[0]) if self.mask else 0)
-        else:
-            my_areas = _get_rectangle_collision_areas(
-                self.bbox_left, self.bbox_top, self.bbox_width,
-                self.bbox_height)
     else:
         colliders = _colliders[:]
         while self in colliders:
@@ -545,63 +580,20 @@ def o_update_collision_lists(self):
             collision_checkers.remove(self)
         _collision_checkers = collision_checkers
 
-        my_areas = []
 
-    if my_areas != self.rd["collision_areas"]:
-        room = sge.game.current_room
-        for area in self.rd["collision_areas"]:
-            if area not in my_areas:
-                if area is None:
-                    area = (None, None)
-                i, j = area
-                if (i is not None and j is not None and
-                        i < len(room.rd["collision_areas"]) and
-                        j < len(room.rd["collision_areas"][i])):
-                    ca = room.rd["collision_areas"][i][j][:]
-                    while self in ca:
-                        ca.remove(self)
-                    room.rd["collision_areas"][i][j] = ca
-                else:
-                    if i is not None or j is not None:
-                        e = "A collision area existed in a {} object, but not in the room!".format(
-                            self.__class__.__name__)
-                        e += "\nAttempted area: ({}, {})".format(i, j)
-                        x = len(room.rd["collision_areas"])
-                        y = len(room.rd["collision_areas"][0]) if x else 0
-                        e += "\nAvailable collision areas: {}x{}".format(x, y)
-                        warnings.warn(e)
+def o_is_other(self, other=None):
+    r = False
 
-                    ca = room.rd["collision_area_void"][:]
-                    while self in ca:
-                        ca.remove(self)
-                    room.rd["collision_area_void"] = ca
+    if other is None:
+        r = True
+    elif isinstance(other, sge.Object):
+        r = self is other
+    elif isinstance(other, (list, tuple, set)):
+        r = self in other
+    elif inspect.isclass(other):
+        r = isinstance(self, other)
 
-        for area in my_areas:
-            if area not in self.rd["collision_areas"]:
-                if area is None:
-                    area = (None, None)
-                i, j = area
-                if (i is not None and j is not None and
-                        i < len(room.rd["collision_areas"]) and
-                        j < len(room.rd["collision_areas"][i])):
-                    ca = room.rd["collision_areas"][i][j][:]
-                    ca.append(self)
-                    room.rd["collision_areas"][i][j] = ca
-                else:
-                    if i is not None or j is not None:
-                        e = "A collision area existed in a {} object, but not in the room!".format(
-                            self.__class__.__name__)
-                        e += "\nAttempted area: ({}, {})".format(i, j)
-                        x = len(room.rd["collision_areas"])
-                        y = len(room.rd["collision_areas"][0]) if x else 0
-                        e += "\nAvailable collision areas: {}x{}".format(x, y)
-                        warnings.warn(e)
-
-                    ca = room.rd["collision_area_void"][:]
-                    ca.append(self)
-                    room.rd["collision_area_void"] = ca
-
-    self.rd["collision_areas"] = my_areas
+    return r
 
 
 def o_detect_collisions(self):
@@ -696,6 +688,57 @@ def o_set_speed(self):
     self.rd["mv_dir"] = math.degrees(math.atan2(-self.rd["yv"], self.rd["xv"]))
 
 
+def r_get_rectangle_object_areas(self, x, y, width, height):
+    # Get a set of object areas a rect is in.
+    xis = int(math.floor(x / self.object_area_width))
+    yis = int(math.floor(y / self.object_area_height))
+    xie = int(math.ceil((x + width) / self.object_area_width))
+    yie = int(math.ceil((y + height) / self.object_area_height))
+
+    areas = set()
+
+    if (self.object_areas and xis < len(self.object_areas) and
+            yis < len(self.object_areas[0]) and xie > 0 and yie > 0):
+        use_void = False
+
+        if xis < 0:
+            xis = 0
+            use_void = True
+        if yis < 0:
+            yis = 0
+            use_void = True
+        if xie > len(self.object_areas):
+            xie = len(self.object_areas)
+            use_void = True
+        if yie > len(self.object_areas[0]):
+            yie = len(self.object_areas[0])
+            use_void = True
+
+        if use_void:
+            areas.add(None)
+
+        for xi in six.moves.range(xis, xie):
+            for yi in six.moves.range(yis, yie):
+                areas.add((xi, yi))
+    else:
+        areas.add(None)
+
+    return areas
+
+
+def r_set_object_areas(self):
+    self.object_areas = []
+    for i in six.moves.range(0, self.width, self.object_area_width):
+        column = [set() for j in six.moves.range(0, self.height,
+                                                 self.object_area_height)]
+        self.object_areas.append(column)
+
+    self.object_area_void = set()
+
+    for obj in self.objects:
+        o_update_object_areas(obj)
+
+
 def r_update_fade(self, complete):
     transition_sprite = self.rd["t_sprite"]
     w = transition_sprite.width
@@ -877,8 +920,8 @@ def r_update_iris_in(self, complete):
         x = w / 2
         y = h / 2
 
-    r = int(math.hypot(max(x, self.width - x), max(y, self.height - y)) *
-            (1 - complete))
+    r = int(math.hypot(max(x, sge.game.width - x),
+                       max(y, sge.game.height - y)) * (1 - complete))
     eraser = sge.Sprite(width=w, height=h)
     eraser_eraser = sge.Sprite(width=w, height=h)
     eraser_eraser.draw_circle(x, y, r, fill=sge.Color((0, 0, 0, 255)))
@@ -904,8 +947,8 @@ def r_update_iris_out(self, complete):
         x = w / 2
         y = h / 2
 
-    r = int(math.hypot(max(x, self.width - x), max(y, self.height - y)) *
-            (1 - complete))
+    r = int(math.hypot(max(x, sge.game.width - x),
+                       max(y, sge.game.height - y)) * complete)
     eraser = sge.Sprite(width=w, height=h)
     eraser.draw_circle(x, y, r, fill=sge.Color((0, 0, 0, 255)))
     transition_sprite.draw_sprite(eraser, 0, 0, 0,
